﻿using System;
using System.IO;
using System.Linq;
using System.Security.Cryptography;

namespace RaDynamics.QrZahlteilApp.Service
{
    internal class AesEncryption : IEncryption
    {
        public byte[] Secret { get; }
        public byte[] IV { get; }

        public AesEncryption(byte[] secret, byte[] iv)
        {
            if (!new[] { 32, 256 }.Contains(secret.Length)) throw new ArgumentOutOfRangeException("Bei AES muss das Secret 32 oder 256 Bytes sein.");
            if (!new[] { 16 }.Contains(iv.Length)) throw new ArgumentOutOfRangeException("Bei AES muss der Initialisierungsvektor (IV) 16 Bytes sein.");
            Secret = secret;
            IV = iv;
        }

        public byte[] Encrypt(string value)
        {
            if (value == null || value.Length <= 0) {
                throw new ArgumentNullException(nameof(value));
            }

            using (var algo = CreateCryptAlgo()) {
                var encryptor = algo.CreateEncryptor(algo.Key, algo.IV);
                using (var ms = new MemoryStream()) {
                    using (var cs = new CryptoStream(ms, encryptor, CryptoStreamMode.Write)) {
                        using (var sw = new StreamWriter(cs)) {
                            sw.Write(value);
                        }
                        return ms.ToArray();
                    }
                }
            }
        }

        public string Decrypt(byte[] value)
        {
            using (var algo = CreateCryptAlgo()) {
                var decryptor = algo.CreateDecryptor(algo.Key, algo.IV);
                using (var ms = new MemoryStream(value)) {
                    using (var cs = new CryptoStream(ms, decryptor, CryptoStreamMode.Read)) {
                        using (var sr = new StreamReader(cs)) {
                            return sr.ReadToEnd();
                        }
                    }
                }
            }
        }

        private Aes CreateCryptAlgo()
        {
            var algo = Aes.Create();
            algo.Key = Secret;
            algo.IV = IV;
            algo.Mode = CipherMode.CBC;
            algo.Padding = PaddingMode.PKCS7;
            return algo;
        }
    }
}
