﻿using System;
using System.Collections.Generic;
using System.Drawing;

namespace RaDynamics.QrZahlteilApp.Service
{
    public class ScannedData
    {
        private readonly List<Image> _pictures = new List<Image>();

        /// <summary>
        /// Typ der gescannten Daten
        /// </summary>
        public ScannedType Type { get; }
        /// <summary>
        /// Der QR-Code der QR-Rechnung oder die EZS Kodierzeile in unveränderter Form.
        /// </summary>
        public string Raw { get; private set; }
        /// <summary>
        /// Vom Benutzer aufgenommene Bilder.
        /// <remarks>Wenn keine Bilder aufgenommen wurden, ist das Array leer.</remarks>
        /// </summary>
        public Image[] Pictures => _pictures.ToArray();
        public byte[] Pdf { get; set; } = new byte[0];
        public bool HasPdf => Pdf?.Length > 0;

        private ScannedData(ScannedType type)
        {
            Type = type;
        }

        public ScannedData(ScannedType type, string raw)
            : this(type)
        {
            Raw = raw ?? throw new ArgumentNullException(nameof(raw));
        }

        public static ScannedData Create(ScannedType type, string rawDecoded)
        {
            return new ScannedData(type) { Raw = rawDecoded };
        }

        public void Add(Image picture)
        {
            if (picture == null) throw new ArgumentNullException(nameof(picture));
            _pictures.Add(picture);
        }
    }
}
