﻿using System;
using System.Drawing;
using System.IO;
using System.Linq;
using System.Security.Cryptography;
using System.Text;

namespace RaDynamics.QrZahlteilApp.Service
{
    internal static class Utils
    {
        public static string FromBase64(string value)
        {
            var base64EncodedBytes = Convert.FromBase64String(value);
            return Encoding.UTF8.GetString(base64EncodedBytes);
        }

        public static byte[] FromBase64String(string value)
        {
            return Convert.FromBase64String(value);
        }

        public static string ToBase64(string value)
        {
            return ToBase64(Encoding.UTF8.GetBytes(value));
        }

        public static string ToBase64(byte[] value)
        {
            return Convert.ToBase64String(value);
        }

        public static Image ToImage(string base64)
        {
            byte[] data = Convert.FromBase64String(base64);
            return Image.FromStream(new MemoryStream(data, 0, data.Length));
        }

        public static byte[] CreateSecret(int length)
        {
            const string valid = "abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ1234567890";
            var res = new StringBuilder();
            using (var rng = new RNGCryptoServiceProvider()) {
                var buffer = new byte[sizeof(uint)];

                while (length-- > 0) {
                    rng.GetBytes(buffer);
                    var num = BitConverter.ToUInt32(buffer, 0);
                    res.Append(valid[(int)(num % (uint)valid.Length)]);
                }
            }

            return Encoding.UTF8.GetBytes(res.ToString());
        }

        public static string ToHex(byte[] value)
        {
            if (value == null) return null;
            return BitConverter.ToString(value).Replace("-", "");
        }
        public static byte[] FromHex(string value)
        {
            if (value == null) throw new ArgumentNullException(nameof(value));
            return Enumerable.Range(0, value.Length).Where(x => x % 2 == 0).Select(x => Convert.ToByte(value.Substring(x, 2), 16)).ToArray();
        }
    }
}
